from datetime import datetime
from typing import List, Optional, Dict, Any

from janis_core import *
from janis_core.types.common_data_types import Boolean, File, Array, String, Int

Runbesst_V0_1_0 = CommandToolBuilder(tool="runBESST", base_command=["runBESST"], inputs=[ToolInput(tag="in_deactivate_pathfinder_module", input_type=Boolean(optional=True), prefix="-y", doc=InputDocumentation(doc="Deactivate pathfinder module for including smaller")), ToolInput(tag="in_fasta_file_containing", input_type=File(optional=True), prefix="-c", doc=InputDocumentation(doc="Fasta file containing contigs.")), ToolInput(tag="in_paths_to_bamfiles", input_type=Array(t=String(), optional=True), prefix="-f", doc=InputDocumentation(doc="Path(s) to bamfile(s).")), ToolInput(tag="in_orientation", input_type=Array(t=String(), optional=True), prefix="-orientation", doc=InputDocumentation(doc="Mapped orientation for each library given with -f\noption. Valid input is either fr (forward reverse\norientation) or rf (reverse forward orientation).")), ToolInput(tag="in_mean_read_length", input_type=Array(t=String(), optional=True), prefix="-r", doc=InputDocumentation(doc="Mean read length of libraries.")), ToolInput(tag="in_mean_insert_size", input_type=Array(t=String(), optional=True), prefix="-m", doc=InputDocumentation(doc="Mean insert size of libraries.")), ToolInput(tag="in_estimated_standard_deviation", input_type=Array(t=String(), optional=True), prefix="-s", doc=InputDocumentation(doc="Estimated standard deviation of libraries.")), ToolInput(tag="in_user_over_value", input_type=String(optional=True), prefix="-z", doc=InputDocumentation(doc="User specified coverage cutoff. (Manually filter out\ncontigs with coverage over this value)")), ToolInput(tag="in_z_min", input_type=String(optional=True), prefix="-z_min", doc=InputDocumentation(doc="User specified coverage cutoff. (Manually filter out\ncontigs with coverage under this value)")), ToolInput(tag="in_maximum_length_ratio", input_type=Int(optional=True), prefix="-a", doc=InputDocumentation(doc="Maximum length ratio for merging of haplotypic")), ToolInput(tag="in_haplotype_detection_function", input_type=Boolean(optional=True), prefix="-g", doc=InputDocumentation(doc="Haplotype detection function, default = off")), ToolInput(tag="in_threshold_value_filter", input_type=Array(t=String(), optional=True), prefix="-T", doc=InputDocumentation(doc="Threshold value filter out reads that are mapped too\nfar apart given insert size.")), ToolInput(tag="in_threshold_value_least", input_type=Array(t=String(), optional=True), prefix="-e", doc=InputDocumentation(doc="Threshold value for the least nr of links that is\nneeded to create an edge. Default for all libs:\nInferred by BESST (see value in output in\nStatistics.txt).")), ToolInput(tag="in_contig_size_threshold", input_type=Array(t=String(), optional=True), prefix="-k", doc=InputDocumentation(doc="Contig size threshold for the library (all contigs\nbelow this size is discarded from the 'large contigs'\nscaffolding, but included in pathfinding). Default:\nSet to same as -T parameter")), ToolInput(tag="in_filter_contigs", input_type=Int(optional=True), prefix="-filter_contigs", doc=InputDocumentation(doc="Contigs under this size is discarded from all\nscaffolding (including pathfinding). they are also not\nincluded in the resulting scaffold output fasta file\nand any of the other files as well as all statistics.\nDefault: All contigs are included")), ToolInput(tag="in_min_mapq", input_type=Int(optional=True), prefix="--min_mapq", doc=InputDocumentation(doc="Lowest mapping quality allowed in order to use the\nread pair (both reads needs to have equal or bigger\nmapq than this value). This value is compared to the\nmapping quality column in the BAM file.")), ToolInput(tag="in_iter", input_type=File(optional=True), prefix="--iter", doc=InputDocumentation(doc="The number of iterations performed in breadth first\nsearch for placing smaller contigs.")), ToolInput(tag="in_score_cut_off", input_type=String(optional=True), prefix="--score_cutoff", doc=InputDocumentation(doc="Only store paths with score larger than score_cutoff.")), ToolInput(tag="in_max_extensions", input_type=Int(optional=True), prefix="--max_extensions", doc=InputDocumentation(doc="Maximum number of (large) scaffolds to search for\npaths extensions with.")), ToolInput(tag="in_no_ilp", input_type=Boolean(optional=True), prefix="--NO_ILP", doc=InputDocumentation(doc="Warning: Do not use this option! This option is\nincluded only for benchmarking purposes of old BESST\nalgorithm. This option will give poor results as it\nmimics earlier versions of BESST.")), ToolInput(tag="in_faster_ilp", input_type=Boolean(optional=True), prefix="--FASTER_ILP", doc=InputDocumentation(doc="Faster but worse performing heuristic solution to\nsolving ILPs. May be used if BESST is too slow.\nHowever, lowering --iter is usually more effective to\nreduce scaffolding time.")), ToolInput(tag="in_print_scores", input_type=Boolean(optional=True), prefix="--print_scores", doc=InputDocumentation(doc="Print BESST scores on edges in the Scaffolding graph.")), ToolInput(tag="in_kmer_size_used", input_type=Int(optional=True), prefix="-K", doc=InputDocumentation(doc="k-mer size used in de brujin graph for obtaining\ncontigs in assembly, default 50")), ToolInput(tag="in_mmer_usted_creating", input_type=Int(optional=True), prefix="-M", doc=InputDocumentation(doc="m-mer usted for creating connection graph. Should be\nset lower than k-mer size")), ToolInput(tag="in_path_output_directory", input_type=File(optional=True), prefix="-o", doc=InputDocumentation(doc="Path to output directory. BESST will create a folder\nnamed 'BESST_output' in the directory given by the\npath.")), ToolInput(tag="in_deactivate_sequencing_duplicates", input_type=Boolean(optional=True), prefix="-d", doc=InputDocumentation(doc="Deactivate sequencing duplicates detection")), ToolInput(tag="in_parallellize_work_load", input_type=Boolean(optional=True), prefix="-q", doc=InputDocumentation(doc="Parallellize work load of path finder module in case\nof multiple processors available.")), ToolInput(tag="in_devel", input_type=Boolean(optional=True), prefix="-devel", doc=InputDocumentation(doc="Run in development mode (bug checking and memory usage\netc.)")), ToolInput(tag="in_plots", input_type=Boolean(optional=True), prefix="-plots", doc=InputDocumentation(doc="Plot coverage, histograms of scores e.t.c.")), ToolInput(tag="in_separate_repeats", input_type=File(optional=True), prefix="--separate_repeats", doc=InputDocumentation(doc="Separates contigs classified as repeats by BESST into\na file 'repeats.fa'. They are not included in the main\nscaffolding output file with this flag specified.")), ToolInput(tag="in_dfs_traversal", input_type=Boolean(optional=True), prefix="--dfs_traversal", doc=InputDocumentation(doc="Depth first search with DP in the contig graph\n(default).")), ToolInput(tag="in_bfs_traversal", input_type=Boolean(optional=True), prefix="--bfs_traversal", doc=InputDocumentation(doc="Choose a breadth first search in the contig graph.\nDefault is the new depth first search with a DP\napproch that seems to outperform previous traversals.\nThis option is kept for evaluation but is not\nreccomended to specify.")), ToolInput(tag="in_max_contig_overlap", input_type=Int(optional=True), prefix="-max_contig_overlap", doc=InputDocumentation(doc="BESST checks for overlapping ends in contigs that are\nadjacent in a scaffold. This parameter sets the\nmaximum identical overlap to search for, default is\n200.")), ToolInput(tag="in_contigs_dot", input_type=String(), position=0, doc=InputDocumentation(doc="--no_score            Statistical scoring is not performed. BESST instead")), ToolInput(tag="in_regions_dot", input_type=String(), position=0, doc=InputDocumentation(doc="-b HAPLTHRESHOLD      Number of standard deviations over mean/2 of coverage"))], outputs=[ToolOutput(tag="out_path_output_directory", output_type=File(optional=True), selector=InputSelector(input_to_select="in_path_output_directory", type_hint=File()), doc=OutputDocumentation(doc="Path to output directory. BESST will create a folder\nnamed 'BESST_output' in the directory given by the\npath.")), ToolOutput(tag="out_separate_repeats", output_type=File(optional=True), selector=InputSelector(input_to_select="in_separate_repeats", type_hint=File()), doc=OutputDocumentation(doc="Separates contigs classified as repeats by BESST into\na file 'repeats.fa'. They are not included in the main\nscaffolding output file with this flag specified."))], container=None, version="v0.1.0")


if __name__ == "__main__":
    # or "cwl"
    Runbesst_V0_1_0().translate("wdl", allow_empty_container=True)

